#include "math.h"
#include <avr/pgmspace.h>
#include "tinymt32.h"


//---------------------------------------------------------------------------
PROGMEM const s8 MathSinTable[] = {
	   0,		/* no.  0 */
	   2,		/* no.  1 */
	   3,		/* no.  2 */
	   5,		/* no.  3 */
	   6,		/* no.  4 */
	   8,		/* no.  5 */
	   9,		/* no.  6 */
	  11,		/* no.  7 */
	  12,		/* no.  8 */
	  14,		/* no.  9 */
	  16,		/* no. 10 */
	  17,		/* no. 11 */
	  19,		/* no. 12 */
	  20,		/* no. 13 */
	  22,		/* no. 14 */
	  23,		/* no. 15 */
	  24,		/* no. 16 */
	  26,		/* no. 17 */
	  27,		/* no. 18 */
	  29,		/* no. 19 */
	  30,		/* no. 20 */
	  32,		/* no. 21 */
	  33,		/* no. 22 */
	  34,		/* no. 23 */
	  36,		/* no. 24 */
	  37,		/* no. 25 */
	  38,		/* no. 26 */
	  39,		/* no. 27 */
	  41,		/* no. 28 */
	  42,		/* no. 29 */
	  43,		/* no. 30 */
	  44,		/* no. 31 */
	  45,		/* no. 32 */
	  46,		/* no. 33 */
	  47,		/* no. 34 */
	  48,		/* no. 35 */
	  49,		/* no. 36 */
	  50,		/* no. 37 */
	  51,		/* no. 38 */
	  52,		/* no. 39 */
	  53,		/* no. 40 */
	  54,		/* no. 41 */
	  55,		/* no. 42 */
	  56,		/* no. 43 */
	  56,		/* no. 44 */
	  57,		/* no. 45 */
	  58,		/* no. 46 */
	  59,		/* no. 47 */
	  59,		/* no. 48 */
	  60,		/* no. 49 */
	  60,		/* no. 50 */
	  61,		/* no. 51 */
	  61,		/* no. 52 */
	  62,		/* no. 53 */
	  62,		/* no. 54 */
	  62,		/* no. 55 */
	  63,		/* no. 56 */
	  63,		/* no. 57 */
	  63,		/* no. 58 */
	  64,		/* no. 59 */
	  64,		/* no. 60 */
	  64,		/* no. 61 */
	  64,		/* no. 62 */
	  64,		/* no. 63 */
	  64,		/* no. 64 */
	  64,		/* no. 65 */
	  64,		/* no. 66 */
	  64,		/* no. 67 */
	  64,		/* no. 68 */
	  64,		/* no. 69 */
	  63,		/* no. 70 */
	  63,		/* no. 71 */
	  63,		/* no. 72 */
	  62,		/* no. 73 */
	  62,		/* no. 74 */
	  62,		/* no. 75 */
	  61,		/* no. 76 */
	  61,		/* no. 77 */
	  60,		/* no. 78 */
	  60,		/* no. 79 */
	  59,		/* no. 80 */
	  59,		/* no. 81 */
	  58,		/* no. 82 */
	  57,		/* no. 83 */
	  56,		/* no. 84 */
	  56,		/* no. 85 */
	  55,		/* no. 86 */
	  54,		/* no. 87 */
	  53,		/* no. 88 */
	  52,		/* no. 89 */
	  51,		/* no. 90 */
	  50,		/* no. 91 */
	  49,		/* no. 92 */
	  48,		/* no. 93 */
	  47,		/* no. 94 */
	  46,		/* no. 95 */
	  45,		/* no. 96 */
	  44,		/* no. 97 */
	  43,		/* no. 98 */
	  42,		/* no. 99 */
	  41,		/* no.100 */
	  39,		/* no.101 */
	  38,		/* no.102 */
	  37,		/* no.103 */
	  36,		/* no.104 */
	  34,		/* no.105 */
	  33,		/* no.106 */
	  32,		/* no.107 */
	  30,		/* no.108 */
	  29,		/* no.109 */
	  27,		/* no.110 */
	  26,		/* no.111 */
	  24,		/* no.112 */
	  23,		/* no.113 */
	  22,		/* no.114 */
	  20,		/* no.115 */
	  19,		/* no.116 */
	  17,		/* no.117 */
	  16,		/* no.118 */
	  14,		/* no.119 */
	  12,		/* no.120 */
	  11,		/* no.121 */
	   9,		/* no.122 */
	   8,		/* no.123 */
	   6,		/* no.124 */
	   5,		/* no.125 */
	   3,		/* no.126 */
	   2,		/* no.127 */
	  -0,		/* no.128 */
	  -2,		/* no.129 */
	  -3,		/* no.130 */
	  -5,		/* no.131 */
	  -6,		/* no.132 */
	  -8,		/* no.133 */
	  -9,		/* no.134 */
	 -11,		/* no.135 */
	 -12,		/* no.136 */
	 -14,		/* no.137 */
	 -16,		/* no.138 */
	 -17,		/* no.139 */
	 -19,		/* no.140 */
	 -20,		/* no.141 */
	 -22,		/* no.142 */
	 -23,		/* no.143 */
	 -24,		/* no.144 */
	 -26,		/* no.145 */
	 -27,		/* no.146 */
	 -29,		/* no.147 */
	 -30,		/* no.148 */
	 -32,		/* no.149 */
	 -33,		/* no.150 */
	 -34,		/* no.151 */
	 -36,		/* no.152 */
	 -37,		/* no.153 */
	 -38,		/* no.154 */
	 -39,		/* no.155 */
	 -41,		/* no.156 */
	 -42,		/* no.157 */
	 -43,		/* no.158 */
	 -44,		/* no.159 */
	 -45,		/* no.160 */
	 -46,		/* no.161 */
	 -47,		/* no.162 */
	 -48,		/* no.163 */
	 -49,		/* no.164 */
	 -50,		/* no.165 */
	 -51,		/* no.166 */
	 -52,		/* no.167 */
	 -53,		/* no.168 */
	 -54,		/* no.169 */
	 -55,		/* no.170 */
	 -56,		/* no.171 */
	 -56,		/* no.172 */
	 -57,		/* no.173 */
	 -58,		/* no.174 */
	 -59,		/* no.175 */
	 -59,		/* no.176 */
	 -60,		/* no.177 */
	 -60,		/* no.178 */
	 -61,		/* no.179 */
	 -61,		/* no.180 */
	 -62,		/* no.181 */
	 -62,		/* no.182 */
	 -62,		/* no.183 */
	 -63,		/* no.184 */
	 -63,		/* no.185 */
	 -63,		/* no.186 */
	 -64,		/* no.187 */
	 -64,		/* no.188 */
	 -64,		/* no.189 */
	 -64,		/* no.190 */
	 -64,		/* no.191 */
	 -64,		/* no.192 */
	 -64,		/* no.193 */
	 -64,		/* no.194 */
	 -64,		/* no.195 */
	 -64,		/* no.196 */
	 -64,		/* no.197 */
	 -63,		/* no.198 */
	 -63,		/* no.199 */
	 -63,		/* no.200 */
	 -62,		/* no.201 */
	 -62,		/* no.202 */
	 -62,		/* no.203 */
	 -61,		/* no.204 */
	 -61,		/* no.205 */
	 -60,		/* no.206 */
	 -60,		/* no.207 */
	 -59,		/* no.208 */
	 -59,		/* no.209 */
	 -58,		/* no.210 */
	 -57,		/* no.211 */
	 -56,		/* no.212 */
	 -56,		/* no.213 */
	 -55,		/* no.214 */
	 -54,		/* no.215 */
	 -53,		/* no.216 */
	 -52,		/* no.217 */
	 -51,		/* no.218 */
	 -50,		/* no.219 */
	 -49,		/* no.220 */
	 -48,		/* no.221 */
	 -47,		/* no.222 */
	 -46,		/* no.223 */
	 -45,		/* no.224 */
	 -44,		/* no.225 */
	 -43,		/* no.226 */
	 -42,		/* no.227 */
	 -41,		/* no.228 */
	 -39,		/* no.229 */
	 -38,		/* no.230 */
	 -37,		/* no.231 */
	 -36,		/* no.232 */
	 -34,		/* no.233 */
	 -33,		/* no.234 */
	 -32,		/* no.235 */
	 -30,		/* no.236 */
	 -29,		/* no.237 */
	 -27,		/* no.238 */
	 -26,		/* no.239 */
	 -24,		/* no.240 */
	 -23,		/* no.241 */
	 -22,		/* no.242 */
	 -20,		/* no.243 */
	 -19,		/* no.244 */
	 -17,		/* no.245 */
	 -16,		/* no.246 */
	 -14,		/* no.247 */
	 -12,		/* no.248 */
	 -11,		/* no.249 */
	  -9,		/* no.250 */
	  -8,		/* no.251 */
	  -6,		/* no.252 */
	  -5,		/* no.253 */
	  -3,		/* no.254 */
	  -2,		/* no.255 */
};

//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
void MathInit(void)
{
	// EMPTY
}
//---------------------------------------------------------------------------
s8 MathSin(u8 deg)
{
	return __LPM(MathSinTable + (deg & 0xff));
}
//---------------------------------------------------------------------------
s8 MathCos(u8 deg)
{
	return __LPM(MathSinTable + ((deg + 192) & 0xff));
}
